@@ -1,30 +1,39 @@
 # -*- coding: utf-8 -*-
-"""celery.backends.cassandra"""
+"""
+    celery.backends.cassandra
+    ~~~~~~~~~~~~~~~~~~~~~~~~~
+
+    Apache Cassandra result store backend.
+
+"""
 from __future__ import absolute_import
 
-try:
+try:  # pragma: no cover
     import pycassa
     from thrift import Thrift
     C = pycassa.cassandra.ttypes
-except ImportError:
-    pycassa = None
+except ImportError:  # pragma: no cover
+    pycassa = None   # noqa
 
 import socket
 import time
 
-from .. import states
-from ..exceptions import ImproperlyConfigured
-from ..utils.timeutils import maybe_timedelta, timedelta_seconds
+from celery import states
+from celery.exceptions import ImproperlyConfigured
+from celery.utils.log import get_logger
+from celery.utils.timeutils import maybe_timedelta, timedelta_seconds
 
 from .base import BaseDictBackend
 
+logger = get_logger(__name__)
+
 
 class CassandraBackend(BaseDictBackend):
     """Highly fault tolerant Cassandra backend.
 
     .. attribute:: servers
 
-        List of Cassandra servers with format: "hostname:port".
+        List of Cassandra servers with format: ``hostname:port``.
 
     :raises celery.exceptions.ImproperlyConfigured: if
         module :mod:`pycassa` is not available.
@@ -46,34 +55,32 @@ class CassandraBackend(BaseDictBackend):
 
         """
         super(CassandraBackend, self).__init__(**kwargs)
-        self.logger = self.app.log.setup_logger(
-                            name="celery.backends.cassandra")
 
-        self.expires = kwargs.get("expires") or maybe_timedelta(
+        self.expires = kwargs.get('expires') or maybe_timedelta(
                                     self.app.conf.CELERY_TASK_RESULT_EXPIRES)
 
         if not pycassa:
             raise ImproperlyConfigured(
-                "You need to install the pycassa library to use the "
-                "Cassandra backend. See https://github.com/pycassa/pycassa")
+                'You need to install the pycassa library to use the '
+                'Cassandra backend. See https://github.com/pycassa/pycassa')
 
         conf = self.app.conf
         self.servers = (servers or
-                        conf.get("CASSANDRA_SERVERS") or
+                        conf.get('CASSANDRA_SERVERS') or
                         self.servers)
         self.keyspace = (keyspace or
-                         conf.get("CASSANDRA_KEYSPACE") or
+                         conf.get('CASSANDRA_KEYSPACE') or
                          self.keyspace)
         self.column_family = (column_family or
-                              conf.get("CASSANDRA_COLUMN_FAMILY") or
+                              conf.get('CASSANDRA_COLUMN_FAMILY') or
                               self.column_family)
-        self.cassandra_options = dict(conf.get("CASSANDRA_OPTIONS") or {},
+        self.cassandra_options = dict(conf.get('CASSANDRA_OPTIONS') or {},
                                       **cassandra_options or {})
         self.detailed_mode = (detailed_mode or
-                              conf.get("CASSANDRA_DETAILED_MODE") or
+                              conf.get('CASSANDRA_DETAILED_MODE') or
                               self.detailed_mode)
-        read_cons = conf.get("CASSANDRA_READ_CONSISTENCY") or "LOCAL_QUORUM"
-        write_cons = conf.get("CASSANDRA_WRITE_CONSISTENCY") or "LOCAL_QUORUM"
+        read_cons = conf.get('CASSANDRA_READ_CONSISTENCY') or 'LOCAL_QUORUM'
+        write_cons = conf.get('CASSANDRA_WRITE_CONSISTENCY') or 'LOCAL_QUORUM'
         try:
             self.read_consistency = getattr(pycassa.ConsistencyLevel,
                                             read_cons)
@@ -87,7 +94,7 @@ class CassandraBackend(BaseDictBackend):
 
         if not self.servers or not self.keyspace or not self.column_family:
             raise ImproperlyConfigured(
-                    "Cassandra backend not configured.")
+                    'Cassandra backend not configured.')
 
         self._column_family = None
 
@@ -104,7 +111,7 @@ class CassandraBackend(BaseDictBackend):
                     Thrift.TException), exc:
                 if time.time() > ts:
                     raise
-                self.logger.warn('Cassandra error: %r. Retrying...', exc)
+                logger.warn('Cassandra error: %r. Retrying...', exc)
                 time.sleep(self._retry_wait)
 
     def _get_column_family(self):
@@ -128,15 +135,16 @@ class CassandraBackend(BaseDictBackend):
         def _do_store():
             cf = self._get_column_family()
             date_done = self.app.now()
-            meta = {"status": status,
-                    "date_done": date_done.strftime('%Y-%m-%dT%H:%M:%SZ'),
-                    "traceback": self.encode(traceback)}
+            meta = {'status': status,
+                    'date_done': date_done.strftime('%Y-%m-%dT%H:%M:%SZ'),
+                    'traceback': self.encode(traceback),
+                    'children': self.encode(self.current_task_children())}
             if self.detailed_mode:
-                meta["result"] = result
+                meta['result'] = result
                 cf.insert(task_id, {date_done: self.encode(meta)},
                           ttl=timedelta_seconds(self.expires))
             else:
-                meta["result"] = self.encode(result)
+                meta['result'] = self.encode(result)
                 cf.insert(task_id, meta,
                           ttl=timedelta_seconds(self.expires))
 
@@ -151,18 +159,19 @@ class CassandraBackend(BaseDictBackend):
                 if self.detailed_mode:
                     row = cf.get(task_id, column_reversed=True, column_count=1)
                     meta = self.decode(row.values()[0])
-                    meta["task_id"] = task_id
+                    meta['task_id'] = task_id
                 else:
                     obj = cf.get(task_id)
                     meta = {
-                        "task_id": task_id,
-                        "status": obj["status"],
-                        "result": self.decode(obj["result"]),
-                        "date_done": obj["date_done"],
-                        "traceback": self.decode(obj["traceback"]),
+                        'task_id': task_id,
+                        'status': obj['status'],
+                        'result': self.decode(obj['result']),
+                        'date_done': obj['date_done'],
+                        'traceback': self.decode(obj['traceback']),
+                        'children': self.decode(obj['children']),
                     }
             except (KeyError, pycassa.NotFoundException):
-                meta = {"status": states.PENDING, "result": None}
+                meta = {'status': states.PENDING, 'result': None}
             return meta
 
         return self._retry_on_error(_do_get)
