@@ -5,29 +5,25 @@
 
     Loader base class.
 
-    :copyright: (c) 2009 - 2012 by Ask Solem.
-    :license: BSD, see LICENSE for more details.
-
 """
 from __future__ import absolute_import
 
+import anyjson
 import importlib
 import os
 import re
-import traceback
-import warnings
 
-from anyjson import deserialize
 from datetime import datetime
 
-from ..datastructures import DictAttribute
-from ..exceptions import ImproperlyConfigured
-from ..utils import (cached_property, get_cls_by_name,
-                     import_from_cwd as _import_from_cwd)
-from ..utils.functional import maybe_list
-from ..utils.encoding import safe_str
+from kombu.utils.encoding import safe_str
 
-BUILTIN_MODULES = frozenset(["celery.task"])
+from celery.datastructures import DictAttribute
+from celery.exceptions import ImproperlyConfigured
+from celery.utils import cached_property
+from celery.utils.imports import import_from_cwd, symbol_by_name
+from celery.utils.functional import maybe_list
+
+BUILTIN_MODULES = frozenset()
 
 ERROR_ENVVAR_NOT_SET = (
 """The environment variable %r is not set,
@@ -49,6 +45,9 @@ class BaseLoader(object):
         * What happens when the worker starts?
             See :meth:`on_worker_init`.
 
+        * What happens when the worker shuts down?
+            See :meth:`on_worker_shutdown`.
+
         * What modules are imported to find tasks?
 
     """
@@ -61,7 +60,7 @@ class BaseLoader(object):
     _conf = None
 
     def __init__(self, app=None, **kwargs):
-        from ..app import app_or_default
+        from celery.app import app_or_default
         self.app = app_or_default(app)
         self.task_modules = set()
 
@@ -79,10 +78,15 @@ class BaseLoader(object):
         pass
 
     def on_worker_init(self):
-        """This method is called when the worker (:program:`celeryd`)
+        """This method is called when the worker (:program:`celery worker`)
         starts."""
         pass
 
+    def on_worker_shutdown(self):
+        """This method is called when the worker (:program:`celery worker`)
+        shuts down."""
+        pass
+
     def on_worker_process_init(self):
         """This method is called when a child process starts."""
         pass
@@ -95,20 +99,25 @@ class BaseLoader(object):
         return importlib.import_module(module, package=package)
 
     def import_from_cwd(self, module, imp=None, package=None):
-        return _import_from_cwd(module,
+        return import_from_cwd(module,
                 self.import_module if imp is None else imp,
                 package=package)
 
     def import_default_modules(self):
-        imports = set(maybe_list(self.conf.get("CELERY_IMPORTS") or ()))
-        return [self.import_task_module(module)
-                    for module in imports | self.builtin_modules]
+        return [self.import_task_module(m)
+            for m in set(maybe_list(self.app.conf.CELERY_IMPORTS))
+                   | set(maybe_list(self.app.conf.CELERY_INCLUDE))
+                   | self.builtin_modules]
 
     def init_worker(self):
         if not self.worker_initialized:
             self.worker_initialized = True
+            self.import_default_modules()
             self.on_worker_init()
 
+    def shutdown_worker(self):
+        self.on_worker_shutdown()
+
     def init_worker_process(self):
         self.on_worker_process_init()
 
@@ -123,26 +132,26 @@ class BaseLoader(object):
     def config_from_object(self, obj, silent=False):
         if isinstance(obj, basestring):
             try:
-                if "." in obj:
-                    obj = get_cls_by_name(obj, imp=self.import_from_cwd)
+                if '.' in obj:
+                    obj = symbol_by_name(obj, imp=self.import_from_cwd)
                 else:
                     obj = self.import_from_cwd(obj)
             except (ImportError, AttributeError):
                 if silent:
                     return False
                 raise
-        if not hasattr(obj, "__getitem__"):
+        if not hasattr(obj, '__getitem__'):
             obj = DictAttribute(obj)
         self._conf = obj
         return True
 
-    def cmdline_config_parser(self, args, namespace="celery",
-                re_type=re.compile(r"\((\w+)\)"),
-                extra_types={"json": deserialize},
-                override_types={"tuple": "json",
-                                "list": "json",
-                                "dict": "json"}):
-        from ..app.defaults import Option, NAMESPACES
+    def cmdline_config_parser(self, args, namespace='celery',
+                re_type=re.compile(r'\((\w+)\)'),
+                extra_types={'json': anyjson.loads},
+                override_types={'tuple': 'json',
+                                'list': 'json',
+                                'dict': 'json'}):
+        from celery.app.defaults import Option, NAMESPACES
         namespace = namespace.upper()
         typemap = dict(Option.typemap, **extra_types)
 
@@ -153,7 +162,7 @@ class BaseLoader(object):
             ## find key/value
             # ns.key=value|ns_key=value (case insensitive)
             key, value = arg.split('=', 1)
-            key = key.upper().replace(".", "_")
+            key = key.upper().replace('.', '_')
 
             ## find namespace.
             # .key=value|_key=value expands to default namespace.
@@ -163,7 +172,7 @@ class BaseLoader(object):
                 # find namespace part of key
                 ns, key = key.split('_', 1)
 
-            ns_key = (ns and ns + "_" or "") + key
+            ns_key = (ns and ns + '_' or '') + key
 
             # (type)value makes cast to custom type.
             cast = re_type.match(value)
@@ -177,7 +186,7 @@ class BaseLoader(object):
                     value = NAMESPACES[ns][key].to_python(value)
                 except ValueError, exc:
                     # display key name in error message.
-                    raise ValueError("%r: %s" % (ns_key, exc))
+                    raise ValueError('%r: %s' % (ns_key, exc))
             return ns_key, value
 
         return dict(map(getarg, args))
@@ -186,22 +195,17 @@ class BaseLoader(object):
             sender=None, to=None, host=None, port=None,
             user=None, password=None, timeout=None,
             use_ssl=False, use_tls=False):
-        try:
-            message = self.mail.Message(sender=sender, to=to,
-                                        subject=safe_str(subject),
-                                        body=safe_str(body))
-            mailer = self.mail.Mailer(host=host, port=port,
-                                      user=user, password=password,
-                                      timeout=timeout, use_ssl=use_ssl,
-                                      use_tls=use_tls)
-            mailer.send(message)
-        except Exception, exc:
-            if not fail_silently:
-                raise
-            warnings.warn(self.mail.SendmailWarning(
-                "Mail could not be sent: %r %r\n%r" % (
-                    exc, {"To": to, "Subject": subject},
-                    traceback.format_stack())))
+        message = self.mail.Message(sender=sender, to=to,
+                                    subject=safe_str(subject),
+                                    body=safe_str(body))
+        mailer = self.mail.Mailer(host=host, port=port,
+                                  user=user, password=password,
+                                  timeout=timeout, use_ssl=use_ssl,
+                                  use_tls=use_tls)
+        mailer.send(message, fail_silently=fail_silently)
+
+    def read_configuration(self):
+        return {}
 
     @property
     def conf(self):
@@ -212,4 +216,4 @@ class BaseLoader(object):
 
     @cached_property
     def mail(self):
-        return self.import_module("celery.utils.mail")
+        return self.import_module('celery.utils.mail')
